# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .hangup_tool import HangupTool
from .retrieval_tool import RetrievalTool
from .inference_embedding_webhook_tool_params import InferenceEmbeddingWebhookToolParams

__all__ = [
    "AssistantTool",
    "Handoff",
    "HandoffHandoff",
    "HandoffHandoffAIAssistant",
    "Transfer",
    "TransferTransfer",
    "TransferTransferTarget",
    "TransferTransferCustomHeader",
    "Refer",
    "ReferRefer",
    "ReferReferTarget",
    "ReferReferCustomHeader",
    "ReferReferSipHeader",
    "SendDtmf",
    "SendMessage",
]


class HandoffHandoffAIAssistant(BaseModel):
    id: str
    """The ID of the assistant to hand off to."""

    name: str
    """Helpful name for giving context on when to handoff to the assistant."""


class HandoffHandoff(BaseModel):
    ai_assistants: List[HandoffHandoffAIAssistant]
    """List of possible assistants that can receive a handoff."""

    voice_mode: Optional[Literal["unified", "distinct"]] = None
    """
    With the unified voice mode all assistants share the same voice, making the
    handoff transparent to the user. With the distinct voice mode all assistants
    retain their voice configuration, providing the experience of a conference call
    with a team of assistants.
    """


class Handoff(BaseModel):
    """
    The handoff tool allows the assistant to hand off control of the conversation to another AI assistant. By default, this will happen transparently to the end user.
    """

    handoff: HandoffHandoff

    type: Literal["handoff"]


class TransferTransferTarget(BaseModel):
    name: Optional[str] = None
    """The name of the target."""

    to: Optional[str] = None
    """The destination number or SIP URI of the call."""


class TransferTransferCustomHeader(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class TransferTransfer(BaseModel):
    from_: str = FieldInfo(alias="from")
    """Number or SIP URI placing the call."""

    targets: List[TransferTransferTarget]
    """The different possible targets of the transfer.

    The assistant will be able to choose one of the targets to transfer the call to.
    """

    custom_headers: Optional[List[TransferTransferCustomHeader]] = None
    """Custom headers to be added to the SIP INVITE for the transfer command."""

    warm_transfer_instructions: Optional[str] = None
    """
    Natural language instructions for your agent for how to provide context for the
    transfer recipient.
    """


class Transfer(BaseModel):
    transfer: TransferTransfer

    type: Literal["transfer"]


class ReferReferTarget(BaseModel):
    name: str
    """The name of the target."""

    sip_address: str
    """The SIP URI to which the call will be referred."""

    sip_auth_password: Optional[str] = None
    """SIP Authentication password used for SIP challenges."""

    sip_auth_username: Optional[str] = None
    """SIP Authentication username used for SIP challenges."""


class ReferReferCustomHeader(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class ReferReferSipHeader(BaseModel):
    name: Optional[Literal["User-to-User", "Diversion"]] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `{{#integration_secret}}test-secret{{/integration_secret}}` to pass the value of
    the integration secret.
    """


class ReferRefer(BaseModel):
    targets: List[ReferReferTarget]
    """The different possible targets of the SIP refer.

    The assistant will be able to choose one of the targets to refer the call to.
    """

    custom_headers: Optional[List[ReferReferCustomHeader]] = None
    """Custom headers to be added to the SIP REFER."""

    sip_headers: Optional[List[ReferReferSipHeader]] = None
    """SIP headers to be added to the SIP REFER.

    Currently only User-to-User and Diversion headers are supported.
    """


class Refer(BaseModel):
    refer: ReferRefer

    type: Literal["refer"]


class SendDtmf(BaseModel):
    send_dtmf: Dict[str, object]

    type: Literal["send_dtmf"]


class SendMessage(BaseModel):
    """
    The send_message tool allows the assistant to send SMS or MMS messages to the end user. The 'to' and 'from' addresses are automatically determined from the conversation context, and the message text is generated by the assistant.
    """

    send_message: Dict[str, object]

    type: Literal["send_message"]


AssistantTool: TypeAlias = Annotated[
    Union[
        InferenceEmbeddingWebhookToolParams, RetrievalTool, Handoff, HangupTool, Transfer, Refer, SendDtmf, SendMessage
    ],
    PropertyInfo(discriminator="type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    DialogflowConnectionCreateResponse,
    DialogflowConnectionUpdateResponse,
    DialogflowConnectionRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDialogflowConnections:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        dialogflow_connection = client.dialogflow_connections.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )
        assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        dialogflow_connection = client.dialogflow_connections.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
            conversation_profile_id="a-VMHLWzTmKjiJw5S6O0-w",
            dialogflow_api="cx",
            environment="development",
            location="global",
        )
        assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.dialogflow_connections.with_raw_response.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = response.parse()
        assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.dialogflow_connections.with_streaming_response.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = response.parse()
            assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.dialogflow_connections.with_raw_response.create(
                connection_id="",
                service_account={
                    "type": "bar",
                    "project_id": "bar",
                    "private_key_id": "bar",
                    "private_key": "bar",
                    "client_email": "bar",
                    "client_id": "bar",
                    "auth_uri": "bar",
                    "token_uri": "bar",
                    "auth_provider_x509_cert_url": "bar",
                    "client_x509_cert_url": "bar",
                },
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        dialogflow_connection = client.dialogflow_connections.retrieve(
            "connection_id",
        )
        assert_matches_type(DialogflowConnectionRetrieveResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.dialogflow_connections.with_raw_response.retrieve(
            "connection_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = response.parse()
        assert_matches_type(DialogflowConnectionRetrieveResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.dialogflow_connections.with_streaming_response.retrieve(
            "connection_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = response.parse()
            assert_matches_type(DialogflowConnectionRetrieveResponse, dialogflow_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.dialogflow_connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        dialogflow_connection = client.dialogflow_connections.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )
        assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        dialogflow_connection = client.dialogflow_connections.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
            conversation_profile_id="a-VMHLWzTmKjiJw5S6O0-w",
            dialogflow_api="cx",
            environment="development",
            location="global",
        )
        assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.dialogflow_connections.with_raw_response.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = response.parse()
        assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.dialogflow_connections.with_streaming_response.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = response.parse()
            assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.dialogflow_connections.with_raw_response.update(
                connection_id="",
                service_account={
                    "type": "bar",
                    "project_id": "bar",
                    "private_key_id": "bar",
                    "private_key": "bar",
                    "client_email": "bar",
                    "client_id": "bar",
                    "auth_uri": "bar",
                    "token_uri": "bar",
                    "auth_provider_x509_cert_url": "bar",
                    "client_x509_cert_url": "bar",
                },
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        dialogflow_connection = client.dialogflow_connections.delete(
            "connection_id",
        )
        assert dialogflow_connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.dialogflow_connections.with_raw_response.delete(
            "connection_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = response.parse()
        assert dialogflow_connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.dialogflow_connections.with_streaming_response.delete(
            "connection_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = response.parse()
            assert dialogflow_connection is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.dialogflow_connections.with_raw_response.delete(
                "",
            )


class TestAsyncDialogflowConnections:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        dialogflow_connection = await async_client.dialogflow_connections.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )
        assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        dialogflow_connection = await async_client.dialogflow_connections.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
            conversation_profile_id="a-VMHLWzTmKjiJw5S6O0-w",
            dialogflow_api="cx",
            environment="development",
            location="global",
        )
        assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.dialogflow_connections.with_raw_response.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = await response.parse()
        assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.dialogflow_connections.with_streaming_response.create(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = await response.parse()
            assert_matches_type(DialogflowConnectionCreateResponse, dialogflow_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.dialogflow_connections.with_raw_response.create(
                connection_id="",
                service_account={
                    "type": "bar",
                    "project_id": "bar",
                    "private_key_id": "bar",
                    "private_key": "bar",
                    "client_email": "bar",
                    "client_id": "bar",
                    "auth_uri": "bar",
                    "token_uri": "bar",
                    "auth_provider_x509_cert_url": "bar",
                    "client_x509_cert_url": "bar",
                },
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        dialogflow_connection = await async_client.dialogflow_connections.retrieve(
            "connection_id",
        )
        assert_matches_type(DialogflowConnectionRetrieveResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.dialogflow_connections.with_raw_response.retrieve(
            "connection_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = await response.parse()
        assert_matches_type(DialogflowConnectionRetrieveResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.dialogflow_connections.with_streaming_response.retrieve(
            "connection_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = await response.parse()
            assert_matches_type(DialogflowConnectionRetrieveResponse, dialogflow_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.dialogflow_connections.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        dialogflow_connection = await async_client.dialogflow_connections.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )
        assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        dialogflow_connection = await async_client.dialogflow_connections.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
            conversation_profile_id="a-VMHLWzTmKjiJw5S6O0-w",
            dialogflow_api="cx",
            environment="development",
            location="global",
        )
        assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.dialogflow_connections.with_raw_response.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = await response.parse()
        assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.dialogflow_connections.with_streaming_response.update(
            connection_id="connection_id",
            service_account={
                "type": "bar",
                "project_id": "bar",
                "private_key_id": "bar",
                "private_key": "bar",
                "client_email": "bar",
                "client_id": "bar",
                "auth_uri": "bar",
                "token_uri": "bar",
                "auth_provider_x509_cert_url": "bar",
                "client_x509_cert_url": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = await response.parse()
            assert_matches_type(DialogflowConnectionUpdateResponse, dialogflow_connection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.dialogflow_connections.with_raw_response.update(
                connection_id="",
                service_account={
                    "type": "bar",
                    "project_id": "bar",
                    "private_key_id": "bar",
                    "private_key": "bar",
                    "client_email": "bar",
                    "client_id": "bar",
                    "auth_uri": "bar",
                    "token_uri": "bar",
                    "auth_provider_x509_cert_url": "bar",
                    "client_x509_cert_url": "bar",
                },
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        dialogflow_connection = await async_client.dialogflow_connections.delete(
            "connection_id",
        )
        assert dialogflow_connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.dialogflow_connections.with_raw_response.delete(
            "connection_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        dialogflow_connection = await response.parse()
        assert dialogflow_connection is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.dialogflow_connections.with_streaming_response.delete(
            "connection_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            dialogflow_connection = await response.parse()
            assert dialogflow_connection is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.dialogflow_connections.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    ChannelZoneListResponse,
    ChannelZoneUpdateResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChannelZones:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        channel_zone = client.channel_zones.update(
            channel_zone_id="channel_zone_id",
            channels=0,
        )
        assert_matches_type(ChannelZoneUpdateResponse, channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.channel_zones.with_raw_response.update(
            channel_zone_id="channel_zone_id",
            channels=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        channel_zone = response.parse()
        assert_matches_type(ChannelZoneUpdateResponse, channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.channel_zones.with_streaming_response.update(
            channel_zone_id="channel_zone_id",
            channels=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            channel_zone = response.parse()
            assert_matches_type(ChannelZoneUpdateResponse, channel_zone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `channel_zone_id` but received ''"):
            client.channel_zones.with_raw_response.update(
                channel_zone_id="",
                channels=0,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        channel_zone = client.channel_zones.list()
        assert_matches_type(SyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        channel_zone = client.channel_zones.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.channel_zones.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        channel_zone = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.channel_zones.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            channel_zone = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncChannelZones:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        channel_zone = await async_client.channel_zones.update(
            channel_zone_id="channel_zone_id",
            channels=0,
        )
        assert_matches_type(ChannelZoneUpdateResponse, channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.channel_zones.with_raw_response.update(
            channel_zone_id="channel_zone_id",
            channels=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        channel_zone = await response.parse()
        assert_matches_type(ChannelZoneUpdateResponse, channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.channel_zones.with_streaming_response.update(
            channel_zone_id="channel_zone_id",
            channels=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            channel_zone = await response.parse()
            assert_matches_type(ChannelZoneUpdateResponse, channel_zone, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `channel_zone_id` but received ''"):
            await async_client.channel_zones.with_raw_response.update(
                channel_zone_id="",
                channels=0,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        channel_zone = await async_client.channel_zones.list()
        assert_matches_type(AsyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        channel_zone = await async_client.channel_zones.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.channel_zones.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        channel_zone = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.channel_zones.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            channel_zone = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[ChannelZoneListResponse], channel_zone, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.portouts import EventListResponse, EventRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEvents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        event = client.portouts.events.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(EventRetrieveResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.portouts.events.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = response.parse()
        assert_matches_type(EventRetrieveResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.portouts.events.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = response.parse()
            assert_matches_type(EventRetrieveResponse, event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.portouts.events.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        event = client.portouts.events.list()
        assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        event = client.portouts.events.list(
            filter={
                "created_at": {
                    "gte": parse_datetime("2021-01-01T00:00:00Z"),
                    "lte": parse_datetime("2021-01-01T00:00:00Z"),
                },
                "event_type": "portout.status_changed",
                "portout_id": "34dc46a9-53ed-4e01-9454-26227ea13326",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.portouts.events.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.portouts.events.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[EventListResponse], event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_republish(self, client: Telnyx) -> None:
        event = client.portouts.events.republish(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert event is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_republish(self, client: Telnyx) -> None:
        response = client.portouts.events.with_raw_response.republish(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = response.parse()
        assert event is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_republish(self, client: Telnyx) -> None:
        with client.portouts.events.with_streaming_response.republish(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = response.parse()
            assert event is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_republish(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.portouts.events.with_raw_response.republish(
                "",
            )


class TestAsyncEvents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.portouts.events.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(EventRetrieveResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.events.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = await response.parse()
        assert_matches_type(EventRetrieveResponse, event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.events.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = await response.parse()
            assert_matches_type(EventRetrieveResponse, event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.portouts.events.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.portouts.events.list()
        assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.portouts.events.list(
            filter={
                "created_at": {
                    "gte": parse_datetime("2021-01-01T00:00:00Z"),
                    "lte": parse_datetime("2021-01-01T00:00:00Z"),
                },
                "event_type": "portout.status_changed",
                "portout_id": "34dc46a9-53ed-4e01-9454-26227ea13326",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.events.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.events.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[EventListResponse], event, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_republish(self, async_client: AsyncTelnyx) -> None:
        event = await async_client.portouts.events.republish(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert event is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_republish(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.events.with_raw_response.republish(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        event = await response.parse()
        assert event is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_republish(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.events.with_streaming_response.republish(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            event = await response.parse()
            assert event is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_republish(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.portouts.events.with_raw_response.republish(
                "",
            )
